function [parameters,ste,robste,LL,BCindex]=PSFA2010(dta,nx,para0,dis,flag,options)

% Estimation of fixed-effect panel SFA model using within-transformation
% method. See Wang and Ho (2010) for more detailed information
% Reference: "Estimating Fixed-Effect Panel Stochastic Frontier Models by
% Model Transformation", Journal of Econometrics, 2010, 157 (2), 289-96.

% USAGE:
%      [parameter,ste]=PSFA2010(dta,nx,para0)
%      [parameter,ste,robste,LL,BCindex]
%                           =PSFA2010(dta,nx,para0,dis,flag,options)

% INPUTS:
%         dta    - a N*T by (K+L+3) matrix of data 
%                   The 1st column is id of DMU (firms/countries);the 2nd
%                   is time variable; the 3rd is the dependent
%                   variable(Y);the 4th -(3+K) are indepent variables (Xs)of
%                   the frontier; the Kth-end are non-stochastic
%                   inefficiency determinants (Zs).
%                   That is to say, ID t Y X1...XK Z1...ZL
%         nx     - The number of independent variables of the frontier (Xs)
%         para0  - starting values for the parameters
%         dis    - The type of inefficiency distirbution, valid types are:
%                      0 if half normal distribution, as default
%                      1 if truncated normal distribution
%         flag   - The type of the frontier, valid types are:
%                      0 if production function, as default
%                      1 if cost function
%        options - options for matlab function "fminunc"

% OUTPUTS: 
%         parameters - A K+L+2(+1 is his=1) vector of parameters.
%                      the 1st-Kth of parameters are coefficients of Xs
%                      the [K+1]th-[K+L]th of parameters are coefficients
%                      of Zs; the [K+L+1]th is the log(sigma_u-square); the
%                      [K+L+1]th is the log(sigma_v-square);(the last is mu
%                      if dis=1). 
%         ste        - A K+L+2(+1 is his=1) vector of standard error.
%         robste     - A K+L+2(+1 is his=1) vector of robust standard error.
%         robste     - the log-likelihood function.
%         BCindex    - BC estimator of efficiency index


% Kerry Du
% kerrydu1986@gmail.com, caike22@126.com
% School of energy, Xiamen University,China

if nargin<4
    dis=0;
end
if nargin<5
    flag=0;
end
if nargin<6
    options=optimset('fminunc');
    options=optimset(options,'Display','iter');
    options=optimset(options , 'MaxFunEvals', 2000);
    options=optimset(options,'LargeScale','off');
end
    

if dis==1 
    if length(para0)~=size(dta,2) 
       error('The length of para0 is not equal to the number of the estimated parameters');
    end
elseif length(para0)~=size(dta,2)-1
    error('The length of para0 is not equal to the number of the estimated parameters');
end
        
    
dta=sortrows(dta,[1 2]);
data=[dta(:,1),dta(:,3:end)];
if flag==1
    data(:,2:nx+2)=-data(:,2:nx+2);
end

[D,id,nt,A]=dw_trans(data(:,1:nx+2));
ind=D(:,1);
y=D(:,2);
x=D(:,3:end);
z=data(:,nx+3:end);
[parameters,LL]=fminunc(@sfa_wlik,para0,options,y,x,z,ind,id,nt,A,dis);
LL=-LL;
[VCVROBUST,~,~,~,hess]=robustvcv(@sfa_wlik,parameters,0,y,x,z,ind,id,nt,A,dis);
VCV=hess^(-1)/length(id);
ste=sqrt(diag(VCVROBUST));
robste=sqrt(diag(VCV));
[mus,sigs2]=sfa_flitter(parameters,y,x,z,ind,id,A,dis);


MUS=[];SIGS2=[];
for jt=1:length(id)
    U1=mus(jt)*ones(nt(jt),1);
    MUS=[MUS;U1];
    U2=sigs2(jt)*ones(nt(jt),1);
    SIGS2=[SIGS2;U2];
end
SIGS=sqrt(SIGS2);
ZZ=data(:,nx+3:end);
nz=size(ZZ,2);
hi=exp(ZZ*parameters(nx+1:nx+nz));
BCindex=((normcdf(MUS./SIGS-hi.*SIGS))./normcdf(MUS./SIGS)).*exp(-hi.*MUS+0.5*(hi.*hi).*SIGS2);


function [D,id,nt,A]=dw_trans(data)

index=data(:,1);
nindiv = length(unique(index));
id = zeros(nindiv,2);    
id(:,1) = unique(index); 

for it=1:nindiv
    id(it,2) = length(find(index ==id(it,1)));
end
nt=id(:,2);
A=cell(nindiv,1);
for it=1:nindiv
    A{it}=eye(id(it,2))-1/id(it,2);
end
A{1}=sparse(A{1});
Md=blkdiag(A{:});
D=Md*data(:,2:end);
D=[index,D];


function [ls,lik]=sfa_wlik(parameters,y,x,z,index,id,nt,A,dis)
parameters=parameters(:);
N=length(id);
n1=length(x(1,:));
n2=length(z(1,:));
beta1=parameters(1:n1);
beta2=parameters(n1+1:n1+n2);
sigv2=exp(parameters(n1+n2+1));
sigu2=exp(parameters(n1+n2+2));
if dis==1
   mu=parameters(n1+n2+3);
else
    mu=0;
end
h=exp(z*beta2);
HH=dw_trans([index,h]);
hi=HH(:,2:end);
e=y-x*beta1;
sigs2=zeros(N,1);
mus=zeros(N,1);
es2=zeros(N,1);
KK=zeros(N,1);
for iter=1:N
    Fi=full(A{iter}*sigv2);
    invFi=pinv(Fi);
    ind=find(index==id(iter));
    sigs2(iter)=1/(hi(ind)'*invFi*hi(ind)+1/sigu2);
    mus(iter)=(mu/sigu2-e(ind)'*invFi*hi(ind))*sigs2(iter);
    es2(iter)=-0.5*e(ind)'*invFi*e(ind);
    KK(iter)=-0.5*(nt(iter)-1)*log(2*pi)-0.5*(nt(iter)-1)*log(sigv2);
end
    lik=KK+es2+0.5*((mus.^2)./sigs2-mu^2/sigu2)...
        +0.5*log(sigs2)+log(normcdf(mus./sqrt(sigs2)))-0.5*log(sigu2)-log(normcdf(mu/sqrt(sigu2)));
    lik=-lik;
    ls=sum(lik);

if isnan(ls)||isreal(ls)==0||isinf(ls)
  idx1=find(isinf(lik)==1);
  idx2=find(isnan(lik)==1);
  idx=[idx1;idx2];
   a=lik;
   a(idx)=1e7;
   ls=sum(a);
end

function [mus,sigs2]=sfa_flitter(parameters,y,x,z,index,id,A,dis)
parameters=parameters(:);
N=length(id);
n1=length(x(1,:));
n2=length(z(1,:));
beta1=parameters(1:n1);
beta2=parameters(n1+1:n1+n2);
sigv2=exp(parameters(n1+n2+1));
sigu2=exp(parameters(n1+n2+2));
if dis==1
   mu=parameters(n1+n2+3);
else
    mu=0;
end
h=exp(z*beta2);
HH=dw_trans([index,h]);
hi=HH(:,2:end);
e=y-x*beta1;
sigs2=zeros(N,1);
mus=zeros(N,1);
for iter=1:N
    Fi=full(A{iter}*sigv2);
    invFi=pinv(Fi);
    ind=find(index==id(iter));
    sigs2(iter)=1/(hi(ind)'*invFi*hi(ind)+1/sigu2);
    mus(iter)=(mu/sigu2-e(ind)'*invFi*hi(ind))*sigs2(iter);
end